package test.projet.ejb.session;

import java.beans.Introspector;
import java.beans.PropertyDescriptor;
import java.lang.reflect.ParameterizedType;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import javax.persistence.Column;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;

import test.projet.ejb.client.enums.PersistenceModeEnum;
import test.projet.ejb.client.exception.CauseException;
import test.projet.ejb.client.exception.ValidationException;
import test.projet.ejb.client.session.EntityUID;
import test.projet.ejb.client.session.JPAFacade;
import test.projet.ejb.client.utils.ColumnAttributes;
import test.projet.ejb.client.utils.Utils;

public class AbstractFacade<T extends EntityUID> implements JPAFacade<T>
{
	@PersistenceContext(name = "MonProjetEJBPU")
	private EntityManager entityManager;
	private Class<?> clazz;
	private Map<String, ColumnAttributes> mapAttributes;

	public AbstractFacade()
	{
		super();
		clazz = (Class<?>) ((ParameterizedType) this.getClass().getGenericSuperclass()).getActualTypeArguments()[0];
	}

	public EntityManager getEntityManager()
	{
		return entityManager;
	}

	public void setEntityManager(EntityManager entityManager)
	{
		this.entityManager = entityManager;
	}

	public Map<String, ColumnAttributes> getMapAttributes() throws Exception
	{
		if (mapAttributes == null)
		{
			mapAttributes = new HashMap<String, ColumnAttributes>();
			for (PropertyDescriptor pdesc : Introspector.getBeanInfo(clazz).getPropertyDescriptors())
			{
				ColumnAttributes columnAttributes = new ColumnAttributes(clazz);
				columnAttributes.setName(pdesc.getName());
				columnAttributes.setColumnClass(pdesc.getPropertyType());
				columnAttributes.setReadMethod(pdesc.getReadMethod());
				columnAttributes.setWriteMethod(pdesc.getWriteMethod());
				for (Column annotation : pdesc.getReadMethod().getAnnotationsByType(Column.class))
				{
					columnAttributes.setLength(annotation.length());
					columnAttributes.setNullable(annotation.nullable());
				}
				mapAttributes.put(columnAttributes.getName(), columnAttributes);
			}
		}
		return mapAttributes;
	}

	@SuppressWarnings("unchecked")
	@Override
	public T get(Integer uid)
	{
		try
		{
			return (T) entityManager.find(clazz, uid);
		} catch (Exception e)
		{
			return null;
		}
	}

	@Override
	public T saveOrUpdate(T entity) throws ValidationException
	{
		if (entity.getUid() == null)
		{
			check(entity, PersistenceModeEnum.CREATE);
			entityManager.persist(entity);
		} else
		{
			check(entity, PersistenceModeEnum.UPDATE);
			entity = entityManager.merge(entity);
		}
		return entity;
	}

	@Override
	public void delete(T entity) throws ValidationException
	{
		entityManager.remove(entity);
	}

	@Override
	public void delete(Integer uid) throws ValidationException
	{
		entityManager.remove(get(uid));
	}

	@Override
	public void check(T entity, PersistenceModeEnum persistenceMode) throws ValidationException
	{
		ValidationException ve = new ValidationException();

		try
		{
			for (ColumnAttributes columnAttributes : getMapAttributes().values())
			{
				Object value = columnAttributes.getReadMethod().invoke(entity, new Object[] {});
				if ("majDate".equals(columnAttributes.getName()))
				{
					columnAttributes.getWriteMethod().invoke(entity, new Date());
				} else
				{
					if (!"uid".equals(columnAttributes.getName()) && !columnAttributes.isNullable() && !Utils.isSet(value))
					{
						ve.addCauseException(new CauseException("exception.field.required", new Object[] { columnAttributes.getLogicalName() }));
					}

					if (value != null && columnAttributes.getColumnClass() == String.class && value.toString().length() > columnAttributes.getLength())
					{
						ve.addCauseException(new CauseException("exception.field.size.exceed", new Object[] { columnAttributes.getLogicalName(), columnAttributes.getLength() }));
					}
				}
			}
		} catch (Exception e)
		{
			ve.addCauseException(new CauseException("exception.unknown", new Object[] { e }));
		}

		if (ve.hasErrors())
		{
			throw ve;
		}
	}

}
