package test.projet.kernel;

import java.io.Serializable;
import java.util.Locale;
import java.util.ResourceBundle;

import javax.faces.application.ConfigurableNavigationHandler;
import javax.faces.application.FacesMessage;
import javax.faces.application.FacesMessage.Severity;
import javax.faces.context.FacesContext;

import test.projet.ejb.client.exception.CauseException;
import test.projet.ejb.client.exception.ValidationException;
import test.projet.ejb.client.utils.Utils;
import test.projet.kernel.enums.ApplicationPagesNamesEnum;

public abstract class AbstractManager implements Serializable
{
	public static final long serialVersionUID = 1;
	public static final String REDIRECT_PART = "faces-redirect";
	public static final String RETURN_POINT_PART = "rp";
	public static final String UID_PART = "uid";
	private String returnPoint;
	private static String contextRoot;

	/*
	 * ----- getter/setter
	 */

	public String getContextRoot()
	{
		if (contextRoot == null)
		{
			contextRoot = "/" + getFacesContext().getExternalContext().getContextName();
		}

		return contextRoot;
	}

	public String getReturnPoint()
	{
		return returnPoint;
	}

	public void setReturnPoint(String returnPoint)
	{
		this.returnPoint = returnPoint;
	}

	public ResourceBundle getResourceBundle()
	{
		FacesContext facesContext = FacesContext.getCurrentInstance();
		return getFacesContext().getApplication().getResourceBundle(facesContext, "msg"); 
	}

	public String getResourceBundleMessage(String key)
	{
		ResourceBundle bundle = getResourceBundle();
		if (bundle.containsKey(key))
		{
			return bundle.getString(key);
		}
		return key;
	}

	public FacesContext getFacesContext()
	{
		return FacesContext.getCurrentInstance();
	}

	public Locale getLocale()
	{
		Locale locale = null;
		try
		{
			locale = getFacesContext().getViewRoot().getLocale();
		}
		catch (Exception e)
		{
			try
			{
				locale = getFacesContext().getExternalContext().getRequestLocale();
			}
			catch (Exception e2)
			{
				locale = new Locale("fr", "FR");
			}
		}
		return locale;
	}


	/*
	 * ----- méthodes
	 */

	/**
	 * Formatage de l'URL de retour à l'appelant
	 * 
	 * @param redirect
	 * @return
	 */
	public String formatToCaller(boolean redirect)
	{
		if (!Utils.isSet(returnPoint))
		{
			return null;
		}
		return formatURL(getPageURL(ApplicationPagesNamesEnum.valueOf(returnPoint)), redirect);
	}

	/**
	 * Formatage de l'URL avec ou sans redirection
	 * 
	 * @param url
	 * @param redirect
	 * @return
	 */
	public String formatURL(String url, boolean redirect)
	{
		if (!Utils.isSet(url))
		{
			return null;
		}
		if (redirect)
		{
			return addURLParameter(new StringBuilder(url), REDIRECT_PART, true).toString();
		}
		return url;
	}

	/**
	 * Ajout d'un paramètre à l'URL
	 * 
	 * @param url
	 * @param attributeName
	 * @param attributeValue
	 * @return
	 */
	public StringBuilder addURLParameter(StringBuilder url, String attributeName, Object attributeValue)
	{
		if (!Utils.isSet(url))
		{
			return null;
		}

		if (!Utils.isSet(attributeName))
		{
			return url;
		}

		if (url.indexOf("?") == -1)
		{
			url.append("?");
		}
		else
		{
			url.append("&");
		}

		url.append(attributeName).append("=");

		if (Utils.isSet(attributeValue))
		{
			url.append(attributeValue.toString());
		}

		return url;
	}

	/**
	 * Formatage de l'URL standard d'appel d'un manager
	 * 
	 * @param url
	 * @param redirect
	 * @param returnPointURL
	 * @param uid
	 * @return
	 */
	public String formatStandardCallURL(String url, boolean redirect, String returnPointURL, Integer uid)
	{
		StringBuilder sb = new StringBuilder();
		sb.append(url);

		if (redirect)
		{
			addURLParameter(sb, REDIRECT_PART, true);
		}

		if (returnPointURL != null)
		{
			addURLParameter(sb, RETURN_POINT_PART, returnPointURL);
		}

		if (uid != null)
		{
			addURLParameter(sb, UID_PART, uid);
		}

		return sb.toString();
	}

	/**
	 * Navigation par programmation
	 * 
	 * @param url
	 * @param redirect
	 * @param returnPointURL
	 * @param uid
	 */
	public void navigateTo(String url, boolean redirect, String returnPointURL, Integer uid)
	{
		ConfigurableNavigationHandler navigationHandler = (ConfigurableNavigationHandler) getFacesContext()
				.getApplication().getNavigationHandler();
		navigationHandler.performNavigation(formatStandardCallURL(url, redirect, returnPointURL, uid));
	}

	/**
	 * Récupération de l'URL associée à l'enum
	 * 
	 * @param page
	 * @return
	 */
	public String getPageURL(ApplicationPagesNamesEnum page)
	{
		String url = getResourceBundleMessage(page.toString());
		return url;
	}

	/**
	 * Envoi d'un message
	 * 
	 * @param text            : Texte ou identifiant de message dans le fichier
	 *                        ApplicationResources.properties
	 * @param severity        : Gravité du message
	 * @param useFlashContext : Utilisation du contexte flash (dans le cadre d'une
	 *                        redirection)
	 */
	public void sendFacesMessage(String text, Severity severity, boolean useFlashContext)
	{
		FacesContext facesContext = getFacesContext();

		facesContext.getExternalContext().getFlash().setKeepMessages(useFlashContext);

		text = getResourceBundleMessage(text);

		if (severity == null)
		{
			severity = FacesMessage.SEVERITY_INFO;
		}

		FacesMessage facesMessage = new FacesMessage(severity, null, text);

		facesContext.addMessage(null, facesMessage);
	}

	/**
	 * Envoi d'un message lié à une Exception
	 * 
	 * @param e               : L'Exception à traiter
	 * @param useFlashContext : Utilisation du contexte flash (dans le cadre d'une
	 *                        redirection)
	 */
	public void sendFacesMessage(Exception e, boolean useFlashContext)
	{
		FacesContext facesContext = getFacesContext();

		facesContext.getExternalContext().getFlash().setKeepMessages(useFlashContext);

		if (e instanceof ValidationException)
		{
			for (CauseException cause : ((ValidationException) e).getCauses())
			{
				FacesMessage facesMessage = new FacesMessage(FacesMessage.SEVERITY_ERROR, cause.toString(),
						formatMessage(cause.getLibelle(), cause.getParameters()));
				facesContext.addMessage(null, facesMessage);
			}
		}
		else
		{
			FacesMessage facesMessage = new FacesMessage(FacesMessage.SEVERITY_ERROR, e.toString(), e.toString());
			facesContext.addMessage(null, facesMessage);
		}
	}

	/**
	 * Formatage d'un message avec d'éventuels paramètres
	 * 
	 * @param message    : Texte du message ou identifiant de message dans le
	 *                   fichier ApplicationResources.properties
	 * @param parameters : Tableau de paramètres ou null
	 * @return Le message formaté
	 */
	public String formatMessage(String message, Object[] parameters)
	{
		String formatted = message;
		int p1, p2 = 0;
		String value = "";

		try
		{
			if (getResourceBundle().containsKey(message))
			{
				formatted = getResourceBundle().getString(message);
			}

			while ((p1 = formatted.indexOf("{")) != -1)
			{
				p2 = formatted.indexOf("}", p1);
				int idx = Integer.valueOf(formatted.substring(p1 + 1, p2));
				if (idx < parameters.length)
				{
					value = parameters[idx].toString();
				}
				else
				{
					value = "";
				}

				value = getResourceBundleMessage(value);
				formatted = formatted.substring(0, p1) + value + formatted.substring(p2 + 1);
			}
		}
		catch (Exception e)
		{

		}
		return formatted;
	}
}
